# main qurator module
import os
import logging
import logging.config
from logging import LoggerAdapter

from .settings import LOGGING_CONFIG, DEPLOY_ENVIRONMENT


def get_logger(name=None, config=None):
    """Top level logger initialiser thingy.

    :name: str identifying logger
    :returns: logging logger object

    """
    if config is not None and name is not None:
        LOGGING_CONFIG['loggers'][name] = config
    logging.config.dictConfig(LOGGING_CONFIG)
    logger = logging.getLogger(name)
    adapter = LoggerAdapter(logger, {"deploy_environment": DEPLOY_ENVIRONMENT})
    adapter.debug(
        "Just created logger with config {!r}".format(LOGGING_CONFIG))
    adapter.debug("Environment: %s" % os.environ.get('QURATOR_SETTINGS_DIR'))
    return adapter



from kombu.mixins import ConsumerMixin
from . import get_logger

logger = get_logger(__name__)


class Worker(ConsumerMixin):

    """Manage server side of RPC connection.

    This code is based on the examples on the Kombu website.
    """

    def __init__(self, connection, consumer):
        """Worker(connection)

        :connection: Connection object
        """
        logger.debug("Called constructor.")
        self.connection = connection
        self.consumer = consumer
        logger.debug("Consumer has queues: {!r}".format(consumer.queues))

    def get_consumers(self, Consumer, channel):
        """Get a set of consumers.

        :Consumer: Consumer object
        :channel: a channel
        :returns: array of Consumer objects
        """
        consumer_set = []
        logger.info("called get_consumers with {!r}".format(self.consumer.queue))
        for i in self.consumer.queues.keys():
            queues = self.consumer.queues[i]
            callbacks = self.consumer.callbacks[i]
            logger.info("Queues: {!r}".format(queues))
            c = Consumer(queues, callbacks=callbacks)
            consumer_set.append(c)
            logger.info("Added consumer: {!r}".format(c))

        logger.info("Processed consumers {!r}".format(consumer_set))
        return consumer_set



import os
import sys

DEPLOY_ENVIRONMENT = os.environ.get('DEPLOY_ENVIRONMENT', 'development')
if DEPLOY_ENVIRONMENT is None:
    raise Exception("Please define the local DEPLOY_ENVIRONMENT.")
RABBITMQ_TRANSPORT_SERVICE_HOST = os.environ.get(
    'RABBITMQ_TRANSPORT_SERVICE_HOST',
    'localhost')
RABBITMQ_TRANSPORT_SERVICE_PORT = os.environ.get(
    'RABBITMQ_TRANSPORT_SERVICE_PORT',
    5672)
RABBITMQ_USER = os.environ.get('RABBITMQ_USER')
RABBITMQ_PASSWORD = os.environ.get('RABBITMQ_PASSWORD')
RABBITMQ_VHOST = os.environ.get('RABBITMQ_VHOST')
RABBITMQ_SSL = os.environ.get('RABBITMQ_SSL', False)

LOGGING_CONFIG = {
    "version": 1,
    "formatters": {
        "simple": {
            "format": "[%(deploy_environment)s] %(pathname)s:line %(lineno)d [%(levelname)s]: %(name)s - %(message)s"
        }
    },
    "handlers": {
        "consoleHandler": {
            "class": "logging.StreamHandler",
            "formatter": "simple",
            "level": "DEBUG",
        }
    },
    "loggers": {
        "root": {
            "level": "DEBUG",
            "handlers": ["consoleHandler"],
            "propagate": 0
        },
        "qurator": {
            "level": "DEBUG",
            "handlers": ["consoleHandler"],
            "propagate": 0
        },
        "qurator.rpc": {
            "level": "DEBUG",
            "handlers": ["consoleHandler"],
            "propagate": 0
        },
        "qurator.utilities": {
            "level": "DEBUG",
            "handlers": ["consoleHandler"],
            "propagate": 0
        }
    }
}

# Copy pasted (with impunity) from domainsage.settings
try:
    if 'QURATOR_SETTINGS_DIR' in os.environ:
        local_settings_dir = os.path.realpath(
            os.path.expanduser(
                os.environ['QURATOR_SETTINGS_DIR']))
        sys.path.insert(0, local_settings_dir)
    from local_settings import *
except ImportError:
    pass

for mq_key in [RABBITMQ_USER, RABBITMQ_PASSWORD, RABBITMQ_VHOST]:
    if mq_key is None:
        raise Exception('Must define RABBITMQ_USER, RABBITMQ_PASSWORD and '
                        'RABBITMQ_VHOST.')

CONN_DICT = {
    'hostname': RABBITMQ_TRANSPORT_SERVICE_HOST,
    'port': RABBITMQ_TRANSPORT_SERVICE_PORT,
    'userid': RABBITMQ_USER,
    'password': RABBITMQ_PASSWORD,
    'ssl': RABBITMQ_SSL,
    'virtual_host': RABBITMQ_VHOST,
    'heartbeat': 2
}



from kombu import Exchange

exchange = Exchange('qurator',
                    type='direct',
                    durable=False)
task_exchange = Exchange('qurator_tasks',
                         type='direct')



import uuid
import subprocess
from distutils.core import setup
from pip.req import parse_requirements
install_reqs = parse_requirements('requirements.txt', session=str(uuid.uuid4()))
reqs = [str(ir.req) for ir in install_reqs]

raw_version = subprocess.Popen(
    "git describe",
    shell=True,
    stdout=subprocess.PIPE).stdout.read()
raw_version = raw_version.decode('utf-8')
version = raw_version.strip()

setup(name='rabbit-qurator',
      author='Travis Holton',
      url='https://github.com/heytrav/rpc-qurator',
      author_email='wtholton@gmail.com',
      install_requires=reqs,
      description='Create RabbitMQ endpoints using decorators.',
      long_description='Create RabbitMQ endpoints for RPC and tasks using decorators based on kombu.',
      version=version,
      packages=['qurator',
                'qurator.rpc',
                'qurator.tests'])
